# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"rest"


import os
import sys
import time


from http.server import HTTPServer, BaseHTTPRequestHandler


from ..default import Default
from ..errors  import Errors, debug
from ..object  import Object
from ..persist import Persist, Workdir
from ..thread  import launch


class Config(Default):

    "Config"

    hostname = "localhost"
    port     = 10102

    def __bla__(self):
        pass

    def __blabla__(self):
        pass


class REST(HTTPServer, Object):

    "REST"

    allow_reuse_address = True
    daemon_thread = True

    def __init__(self, *args, **kwargs):
        HTTPServer.__init__(self, *args, **kwargs)
        Object.__init__(self)
        self.host = args[0]
        self._last = time.time()
        self._starttime = time.time()
        self._status = "start"

    def exit(self):
        "shutdown server."
        self._status = ""
        time.sleep(0.2)
        self.shutdown()

    def start(self):
        "start server/"
        self._status = "ok"
        self.serve_forever()

    def request(self):
        "handler request."
        self._last = time.time()

    def error(self, request, addr):
        "handle error."
        exctype, excvalue, _tb = sys.exc_info()
        exc = exctype(excvalue)
        Errors.add(exc)
        if request:
            debug(f'{addr} {excvalue}')


class RESTHandler(BaseHTTPRequestHandler):

    "RESTHandler"

    def setup(self):
        "setup request."
        BaseHTTPRequestHandler.setup(self)
        self._ip = self.client_address[0]
        self._size = 0

    def send(self, txt):
        "send text."
        self.wfile.write(bytes(txt, "utf-8"))
        self.wfile.flush()

    def write_header(self, htype='text/plain'):
        "write a header."
        self.send_response(200)
        self.send_header('Content-type', f'{htype}; charset="utf-8"')
        self.send_header('Server', "1")
        self.end_headers()

    def do_GET(self): # pylint: disable=C0103
        "handle GET."
        if self.path == "/":
            self.write_header("text/html")
            txt = ""
            for fnm in Persist.fns():
                txt += f'<a href="http://{Config.hostname}:{Config.port}/{fnm}">{fnm}</a>\n'
            self.send(html(txt.strip()))
            return
        fnm = Workdir.workdir + os.sep + "store" + os.sep + self.path
        try:
            with open(fnm, "r", encoding="utf-8") as file:
                txt = file.read()
                self.write_header("txt/plain")
                self.send(html(txt))
        except (TypeError, FileNotFoundError, IsADirectoryError) as ex:
            self.send_response(404)
            Errors.add(ex)
            self.end_headers()

    def log(self, code):
        "log access."
        debug(f"{self.address_string()} code {code} path {self.path}")


def html(txt):
    "html template."
    return f"<!doctype html><html>{txt}</html>"


"runtime"


def init():
    "start object server."
    result = None
    try:
        result = REST((Config.hostname, int(Config.port)), RESTHandler)
    except OSError:
        pass
    if result:
        launch(result.start)
    return result



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"timer"


import datetime
import re
import time as ttime


from ..broker  import Broker
from ..client  import Client
from ..event   import Event
from ..object  import update
from ..persist import Persist, find, sync
from ..thread  import launch
from ..timer   import Timer
from ..utils   import laps


MONTHS = [
    'Bo',
    'Jan',
    'Feb',
    'Mar',
    'Apr',
    'May',
    'Jun',
    'Jul',
    'Aug',
    'Sep',
    'Oct',
    'Nov',
    'Dec'
]


FORMATS = [
    "%Y-%m-%d",
    "%d-%m-%Y",
    "%d-%m",
    "%m-%d",
]


class NoDate(Exception):

    "NoDate"


def extract_date(daystr):
    "extract date from string."
    res = None
    for fmt in FORMATS:
        try:
            res = ttime.mktime(ttime.strptime(daystr, fmt))
            break
        except ValueError:
            res = None
    return res


def get_day(daystr):
    "return day from string."
    day = None
    month = None
    yea = None
    try:
        ymdre = re.search(r'(\d+)-(\d+)-(\d+)', daystr)
        if ymdre:
            (day, month, yea) = ymdre.groups()
    except ValueError:
        try:
            ymre = re.search(r'(\d+)-(\d+)', daystr)
            if ymre:
                (day, month) = ymre.groups()
                yea = ttime.strftime("%Y", ttime.localtime())
        except Exception as ex:
            raise NoDate(daystr) from ex
    if day:
        day = int(day)
        month = int(month)
        yea = int(yea)
        date = f"{day} {MONTHS[month]} {yea}"
        return ttime.mktime(ttime.strptime(date, r"%d %b %Y"))
    raise NoDate(daystr)


def get_hour(daystr):
    "return hour from string."
    try:
        hmsre = re.search(r'(\d+):(\d+):(\d+)', str(daystr))
        hours = 60 * 60 * (int(hmsre.group(1)))
        hoursmin = hours  + int(hmsre.group(2)) * 60
        hmsres = hoursmin + int(hmsre.group(3))
    except AttributeError:
        pass
    except ValueError:
        pass
    try:
        hmre = re.search(r'(\d+):(\d+)', str(daystr))
        hours = 60 * 60 * (int(hmre.group(1)))
        hmsres = hours + int(hmre.group(2)) * 60
    except AttributeError:
        return 0
    except ValueError:
        return 0
    return hmsres


def get_time(txt):
    "parse full time string."
    try:
        target = get_day(txt)
    except NoDate:
        target = to_day(today())
    hour =  get_hour(txt)
    if hour:
        target += hour
    return target


def parse_time(txt):
    "parse time from string."
    seconds = 0
    target = 0
    txt = str(txt)
    for word in txt.split():
        if word.startswith("+"):
            seconds = int(word[1:])
            return ttime.time() + seconds
        if word.startswith("-"):
            seconds = int(word[1:])
            return ttime.time() - seconds
    if not target:
        try:
            target = get_day(txt)
        except NoDate:
            target = to_day(today())
        hour =  get_hour(txt)
        if hour:
            target += hour
    return target


def to_day(daystr):
    "parse day from string."
    previous = ""
    line = ""
    daystr = str(daystr)
    res = None
    for word in daystr.split():
        line = previous + " " + word
        previous = word
        try:
            res = extract_date(line.strip())
            break
        except ValueError:
            res = None
        line = ""
    return res


def today():
    "return date."
    return str(datetime.datetime.today()).split()[0]


"commands"


def tmr(event):
    "add a timer."
    result = ""
    if not event.rest:
        nmr = 0
        for _fn, obj in find('timer'):
            lap = float(obj.time) - ttime.time()
            if lap > 0:
                event.reply(f'{nmr} {obj.txt} {laps(lap)}')
                nmr += 1
        return result
    seconds = 0
    line = ""
    for word in event.args:
        if word.startswith("+"):
            try:
                seconds = int(word[1:])
            except (ValueError, IndexError):
                event.reply(f"{seconds} is not an integer")
                return result
        else:
            line += word + " "
    if seconds:
        target = ttime.time() + seconds
    else:
        try:
            target = get_day(event.rest)
        except NoDate:
            target = to_day(today())
        hour =  get_hour(event.rest)
        if hour:
            target += hour
    if not target or ttime.time() > target:
        event.reply("already passed given time.")
        return result
    event.time = target
    diff = target - ttime.time()
    event.reply("ok " +  laps(diff))
    event.result = []
    event.result.append(event.rest)
    timer = Timer(diff, event.show, thrname=event.cmd)
    update(timer, event)
    sync(timer)
    launch(timer.start)
    return result


"runtime"


def init():
    "start timers."
    for _fn, obj in find("timer"):
        if "time" not in obj:
            continue
        diff = float(obj.time) - ttime.time()
        if diff > 0:
            bot = Broker.first()
            evt = Event()
            update(evt, obj)
            evt.orig = object.__repr__(bot)
            timer = Timer(diff, evt.show)
            launch(timer.start)


"register"


Client.add(tmr)
Persist.add(Timer)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"todo list"


import time


from dataclasses import dataclass


from ..client  import Client
from ..object  import Object
from ..persist import Persist, fntime, find, sync
from ..utils   import laps


class NoDate(Exception):

    "NoDate"


@dataclass
class Todo(Object):

    "Todo"

    txt: str = ''


def dne(event):
    "mark todo as done."
    if not event.args:
        event.reply("dne <txt>")
        return
    selector = {'txt': event.args[0]}
    nmr = 0
    for fnm, obj in find('todo', selector):
        nmr += 1
        obj.__deleted__ = True
        sync(obj, fnm)
        event.reply('ok')
        break
    if not nmr:
        event.reply("nothing todo")


def tdo(event):
    "add a todo."
    if not event.rest:
        nmr = 0
        for fnm, obj in find('todo'):
            lap = laps(time.time()-fntime(fnm))
            event.reply(f'{nmr} {obj.txt} {lap}')
            nmr += 1
        if not nmr:
            event.reply("no todo")
        return
    obj = Todo()
    obj.txt = event.rest
    sync(obj)
    event.reply('ok')


"register"


Client.add(dne)
Client.add(tdo)
Persist.add(Todo)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"event"


import threading


from .default import Default


class Event(Default):

    "Event"

    def __init__(self):
        Default.__init__(self)
        self._ready  = threading.Event()
        self.done    = False
        self.orig    = None
        self.result  = []
        self.thr    = None
        self.txt     = ""
        self.type    = "event"

    def ready(self):
        "event is ready."
        self._ready.set()

    def reply(self, txt):
        "add text to the result"
        self.result.append(txt)

    def wait(self, sec=None):
        "wait for event to be ready."
        if self.thr:
            self.thr.join()
        self._ready.wait(sec)
        return self.result


"interface"


def __dir__():
    return (
        'Event',
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"genocide model of the netherlands"


import datetime
import time


from ..broker   import Broker
from ..client   import Client
from ..event    import Event
from ..object   import Object, construct, keys
from ..repeater import Repeater
from ..thread   import launch
from ..utils    import laps


def __dir__():
    return (
            'init',
            'now'
           )


def init():
    "start genocide model."
    for key in keys(oorzaken):
        val = getattr(oorzaken, key, None)
        if val and int(val) > 10000:
            evt = Event()
            evt.txt = ""
            evt.rest = key
            sec = seconds(val)
            repeater = Repeater(sec, cbstats, evt, thrname=aliases.get(key))
            repeater.start()
    launch(daily, name="daily")


DAY = 24*60*60
YEAR = 365*DAY
SOURCE = "https://github.com/bthate/genocide"
STARTDATE = "2020-01-01 00:00:00"
STARTTIME = time.mktime(time.strptime(STARTDATE, "%Y-%m-%d %H:%M:%S"))


oor = """"Totaal onderliggende doodsoorzaken (aantal)";
         "1 Infectieuze en parasitaire ziekten/Totaal infectieuze en parasitaire zktn (aantal)";
         "1 Infectieuze en parasitaire ziekten/1.1 Tuberculose (aantal)";
         "1 Infectieuze en parasitaire ziekten/1.2 Meningokokkeninfecties (aantal)";
         "1 Infectieuze en parasitaire ziekten/1.3 Virale hepatitis (aantal)";
         "1 Infectieuze en parasitaire ziekten/1.4 AIDS (aantal)";
         "1 Infectieuze en parasitaire ziekten/1.5 Ov. infectieuze en parasitaire zktn (aantal)";
         "2 Nieuwvormingen/Totaal nieuwvormingen (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/Totaal kwaadaardige nieuwvormingen (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.1 Kw. nv. van lip, mond en keel (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.2 Kw. nv. van slokdarm (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.3 Kw. nv. van maag (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.4 Kw. nv. van dikke darm (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.5 Kw. nv. van endeldarm en anus (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.6 Kw. nv. lever en intrah. galwegen (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.7 Kw. nv. van galblaas en galwegen (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.8 Kw. nv. van alvleesklier (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.9 Kw. nv. van strottenhoofd (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.10 Kw. nv. van luchtpijp en long (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.11 Melanoom van huid (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.12 Kw. nv. van borst (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.13 Kw. nv. van baarmoederhals (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.14 Kw. nv. van baarmoederlichaam (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.15 Kw. nv. van eierstok (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.16 Kw. nv. van prostaat (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.17 Kw. nv. nier, behalve nierbekken (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.18 Kw. nv. van urineblaas (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.19 Kw. nv. lymf. en bloedv. weefsel (aantal)";
         "2 Nieuwvormingen/2.1 Kwaadaardige nieuwvormingen/2.1.20 Ov. kwaadaardige nieuwvormingen (aantal)";
         "2 Nieuwvormingen/2.2 Overige nieuwvormingen (aantal)";
         "3 Zktn bloed, bloedvormende organen en.. (aantal)";
         "4 Endocriene, voedings-, stofwiss. zktn/Totaal endocriene, voedings-, stofwiss.. (aantal)";
         "4 Endocriene, voedings-, stofwiss. zktn/4.1 Suikerziekte (aantal)";
         "4 Endocriene, voedings-, stofwiss. zktn/4.2 Ov. endocriene, voedings-, stofwis.. (aantal)";
         "5 Psychische en gedragsstoornissen/Totaal psychische stoornissen (aantal)";
         "5 Psychische en gedragsstoornissen/5.1 Psychische stoornissen door alcohol (aantal)";
         "5 Psychische en gedragsstoornissen/5.2 Psychische stoornissen drugs en vl.. (aantal)";
         "5 Psychische en gedragsstoornissen/5.3 Overige psychische stoornissen (aantal)";
         "6 Ziekten van zenuwstelsel en zintuigen/Totaal ziekten zenuwstelsel en zintuigen (aantal)";
         "6 Ziekten van zenuwstelsel en zintuigen/6.1 Hersenvliesontsteking (aantal)";
         "6 Ziekten van zenuwstelsel en zintuigen/6.2 Ziekte van Parkinson (aantal)";
         "6 Ziekten van zenuwstelsel en zintuigen/6.3 Ov. zktn zenuwstelsel en zintuigen (aantal)";
         "7 Ziekten van hart en vaatstelsel/Totaal ziekten van hart en vaatstelsel (aantal)";
         "7 Ziekten van hart en vaatstelsel/7.1 Ziekten van de kransvaten/Totaal ziekten van de kransvaten (aantal)";
         "7 Ziekten van hart en vaatstelsel/7.1 Ziekten van de kransvaten/7.1.1 Acuut hartinfarct (aantal)";
         "7 Ziekten van hart en vaatstelsel/7.1 Ziekten van de kransvaten/7.1.2 Overige ziekten van de kransvaten (aantal)";
         "7 Ziekten van hart en vaatstelsel/7.2 Overige hartziekten (aantal)";
         "7 Ziekten van hart en vaatstelsel/7.3 Hersenvaatletsels (aantal)";
         "7 Ziekten van hart en vaatstelsel/7.4 Overige ziekten hart en vaatstelsel (aantal)";
         "8 Ziekten van de ademhalingsorganen/Totaal ziekten van de ademhalingsorganen (aantal)";
         "8 Ziekten van de ademhalingsorganen/8.1 Griep (aantal)";
         "8 Ziekten van de ademhalingsorganen/8.2 Longontsteking (aantal)";
         "8 Ziekten van de ademhalingsorganen/8.3 Chron. aand. onderste luchtwegen/Totaal chronische aand. onderste lucht.. (aantal)";
         "8 Ziekten van de ademhalingsorganen/8.3 Chron. aand. onderste luchtwegen/8.3.1 Astma (aantal)";
         "8 Ziekten van de ademhalingsorganen/8.3 Chron. aand. onderste luchtwegen/8.3.2 Ov. chron. aand. onderste luchtw.. (aantal)";
         "8 Ziekten van de ademhalingsorganen/8.4 Overige ziekten ademhalingsorganen (aantal)";
         "9 Ziekten van de spijsverteringsorganen/Totaal ziekten spijsverteringsorganen (aantal)";
         "9 Ziekten van de spijsverteringsorganen/9.1 Maagzweer en zweer aan twaalfvinge.. (aantal)";
         "9 Ziekten van de spijsverteringsorganen/9.2 Chronische leveraandoeningen/Totaal chronische leveraandoeningen (aantal)";
         "9 Ziekten van de spijsverteringsorganen/9.2 Chronische leveraandoeningen/9.2.1 Chronische leveraand. alcohol (aantal)";
         "9 Ziekten van de spijsverteringsorganen/9.2 Chronische leveraandoeningen/9.2.2 Ov. chronische leveraandoeningen (aantal)";
         "9 Ziekten van de spijsverteringsorganen/9.3 Ov. ziekten spijsverteringsorganen (aantal)";
         "10 Ziekten van huid en subcutis (aantal)";
         "11 Ziekten botspierstelsel en bindweef../Totaal ziekten spieren, beend., bindwfsl (aantal)";
         "11 Ziekten botspierstelsel en bindweef../11.1 Reumatoïde artritis en artrose (aantal)";
         "11 Ziekten botspierstelsel en bindweef../11.2 Ov. zktn spieren, beend., bindwfsl (aantal)";
         "12 Ziekten van urogenitaal stelsel/Totaal zktn urinewegen en gesl. organen (aantal)";
         "12 Ziekten van urogenitaal stelsel/12.1 Ziekten van nier en urineleider (aantal)";
         "12 Ziekten van urogenitaal stelsel/12.2 Ov. zktn urinewegen en gesl.organen (aantal)";
         "13 Zwangerschap, bevalling en kraambed. (aantal)";
         "14 Aandoeningen v.d. perinatale periode (aantal)";
         "15 Aangeboren afwijkingen/Totaal aangeboren afwijkingen (aantal)";
         "15 Aangeboren afwijkingen/15.1 Aangeboren afw. zenuwstelsel (aantal)";
         "15 Aangeboren afwijkingen/15.2 Aangeboren afw. hart en bloedvaten (aantal)";
         "15 Aangeboren afwijkingen/15.3 Overige aangeboren afwijkingen (aantal)";
         "16 Sympt., afwijkende klinische bevind../Totaal symp. en onvoll. omschr. ziekte.. (aantal)";
         "16 Sympt., afwijkende klinische bevind../16.1 Wiegendood (aantal)";
         "16 Sympt., afwijkende klinische bevind../16.2 Onvoll. omschr. en onbek. oorzaken (aantal)";
         "16 Sympt., afwijkende klinische bevind../16.3 Ov. symptomen en onvolledig omsch.. (aantal)";
         "17 Uitwendige doodsoorzaken/Totaal uitwendige doodsoorzaken (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/Totaal ongevallen (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/17.1.1 Vervoersongevallen/Totaal vervoersongevallen (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/17.1.1 Vervoersongevallen/17.1.1.1 Wegverkeersongevallen (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/17.1.1 Vervoersongevallen/17.1.1.2 Overige vervoersongevallen (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/17.1.2 Accidentele val (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/17.1.3 Accidentele verdrinking (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/17.1.4 Accidentele vergiftiging (aantal)";
         "17 Uitwendige doodsoorzaken/17.1 Ongevallen/17.1.5 Overige ongevallen (aantal)";
         "17 Uitwendige doodsoorzaken/17.2 Zelfdoding (aantal)";
         "17 Uitwendige doodsoorzaken/17.3 Moord en doodslag (aantal)";
         "17 Uitwendige doodsoorzaken/17.4 Gebeurtenissen opzet onbekend (aantal)";
         "17 Uitwendige doodsoorzaken/17.5 Overige uitwendige doodsoorzaken (aantal)";
         "18 COVID-19 (Coronavirus ziekte 19)/18 Totaal COVID-19 (Coronavirus 19) (aantal)";
         "18 COVID-19 (Coronavirus ziekte 19)/18.1 Vastgestelde COVID-19 (aantal)";
         "18 COVID-19 (Coronavirus ziekte 19)/18.2 Vermoedelijke COVID-19 (aantal)""".split(";")


aantal = """
          168678;
          2974;
          32;
          1;
          34;
          23;
          2884;
          47089;
          45103;
          690;
          2013;
          1150;
          3395;
          1235;
          956;
          452;
          2942;
          205;
          10080;
          811;
          3083;
          230;
          560;
          1032;
          3006;
          923;
          1359;
          3636;
          7345;
          1986;
          560;
          3646;
          2799;
          847;
          11682;
          531;
          61;
          11090;
          8387;
          50;
          1792;
          6545;
          36622;
          8037;
          4718;
          3319;
          12682;
          8850;
          7053;
          10503;
          295;
          2726;
          5776;
          146;
          5630;
          1706;
          4882;
          138;
          948;
          433;
          515;
          3796;
          323;
          1067;
          333;
          734;
          3248;
          1958;
          1290;
          2;
          390;
          436;
          42;
          87;
          307;
          7664;
          11;
          4436;
          3217;
          9030;
          6433;
          633;
          590;
          43;
          5234;
          107;
          238;
          221;
          1823;
          107;
          28;
          639;
          20173;
          17495;
          2678
         """.split(";")


aliases = {}
aliases["Nieuwvormingen"] = "cancer"
aliases["Hart en vaatstelsel"] = "hart disease"
aliases["Psychische en gedragsstoornissen"] = "mental illness"
aliases["Ademhalingsorganen"] = "breathing"
aliases["Uitwendige doodsoorzaken"] = "externals"
aliases["Zenuwstelsel en zintuigen"] = "nerves"
aliases["Afwijkende klinische bevindingen"] = "other diseases NOS"
aliases["Spijsverteringsorganen"] = "stomach disease"
aliases["Endocriene, voedings-, stofwisseling"] = "metabolism disease"
aliases["Urogenitaal stelsel"] = "kidney disease"
aliases["Infectieuze en parasitaire ziekten"] = "infectious disease"
aliases["Botspierstelsel en bindweefsel"] = "muscle disease"
aliases["Bloed, bloedvormende organen"] = "blood disease"
aliases["Aangeboren afwijkingen"] = "birth defect"
aliases["Huid en subcutis"] = "skin disease"
aliases["Zwangerschap"] = "pregnancy"
aliases["Suicide"] = "suicide"


demo = Object()
demo.gehandicapten = 2000000
demo.ggz = 800000
demo.population = 17440000
demo.part = 7000000000 / demo.population


jaar = {}
jaar["WvGGZ"] = 14206
jaar["Pvp"] = 20088
jaar["Wzd"] = 25000
jaar["Wfz"] = 23820
jaar["totaal"] = 168678


oorzaak = Object()
construct(oorzaak, zip(oor, aantal))
oorzaken = Object()


def getalias(txt):
    "teturn matching alias."
    result = None
    for key, value in aliases.items():
        if txt.lower() in key.lower():
            result = value
            break
    return result


def getday():
    "return timestamp of current day."
    day = datetime.datetime.now()
    day = day.replace(hour=0, minute=0, second=0, microsecond=0)
    return day.timestamp()


def getnr(name):
    "return number of deaths by name."
    for k in keys(oorzaken):
        if name.lower() in k.lower():
            return int(getattr(oorzaken, k))
    return 0


def seconds(nrs):
    "calculate sedconds"
    if not nrs:
        return nrs
    return 60*60*24*365 / float(nrs)



def iswanted(k, line):
    "see whether an item is wanted."
    for word in line:
        if word in k:
            return True
    return False


def daily():
    "run a callback daily."
    while 1:
        time.sleep(24*60*60)
        cbnow(Event())


def hourly():
    "run a callback hourly."
    while 1:
        time.sleep(60*60)
        cbnow(Event())


def cbnow(evt):
    "check status now callback."
    if not evt:
        evt = Event()
    delta = time.time() - STARTTIME
    txt = laps(delta) + " "
    for name in sorted(keys(oorzaken), key=lambda x: seconds(getnr(x))):
        needed = seconds(getnr(name))
        if needed > 60*60:
            continue
        nrtimes = int(delta/needed)
        txt += f"{getalias(name)}: {nrtimes}"
    txt += " http://genocide.rtfd.io"
    for bot in Broker.all():
        if "announce" in dir(bot):
            bot.announce(txt)


def cbstats(evt):
    "callback showing stats."
    name = evt.rest or "Psych"
    needed = seconds(getnr(name))
    if needed:
        delta = time.time() - STARTTIME
        nrt = int(delta/needed)
        nry = int(YEAR/needed)
        nrd = int(DAY/needed)
        delta2 = time.time() - getday()
        this = int(delta2/needed)
        txt = f"#{nrt} died from {getalias(name)} ({this}/{nrd}) every {laps(needed)} ({nry}/year)"
        for bot in Broker.all():
            bot.announce(txt)


def now(event):
    "showing number of psychiatric patients victims."
    name = event.rest or "Psych"
    needed = seconds(getnr(name))
    if needed:
        delta = time.time() - STARTTIME
        txt = laps(delta) + " "
        nrt = int(delta/needed)
        nrd = int(DAY/needed)
        this = int(DAY % needed)
        txt = f"#{nrt} died from {getalias(name)} ({this}/{nrd}) every {laps(needed)}"
        event.reply(txt)
    else:
        event.reply("not needed")


"register"


Client.add(now)


"runtime"


def boot():
    "format model data."
    _nr = -1
    for key in keys(oorzaak):
        _nr += 1
        if _nr == 0:
            continue
        if key.startswith('"'):
            key = key[1:]
        lines = key.split("/")
        if len(lines) > 1 and not lines[1].startswith("Totaal"):
            continue
        atl = lines[0].replace('(aantal)"', "")
        atl = atl.replace("Ziekten van de", "")
        atl = atl.replace("Ziekten van", "")
        atl = atl.replace("Ziekten", "")
        atl = atl.replace("Zktn", "")
        atl = atl.replace("zktn", "")
        atl = atl.replace("en..", "")
        atl = atl.replace("..", "")
        atl = atl.replace("bindweef", "bindweefsel")
        atl = atl.replace("bevind", "bevindingen")
        atl = atl.replace("stofwiss.", "stofwisseling")
        atl = atl.replace("Sympt.,", "")
        atl = atl.replace(", bevalling en kraambed. ", "")
        atl = atl.replace("Aandoeningen v.d. ", "")
        nms = " ".join(atl.split()[1:]).capitalize()
        nms = nms.strip()
        setattr(oorzaken, nms, aantal[_nr])


boot()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105
 

"cmd"


from ..client import Client


def cmd(event):
    "list of commands."
    event.reply(",".join(sorted(list(Client.cmds))))


"register"


Client.add(cmd)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"broker"


from .object import Object, keys, values


rpr = object.__repr__


class Broker:

    "Broker"

    objs = Object()

    @staticmethod
    def add(obj):
        "add an object to the broker."
        setattr(Broker.objs, rpr(obj), obj)

    @staticmethod
    def all():
        "return all objects."
        return values(Broker.objs)

    @staticmethod
    def first():
        "return first object."
        for key in keys(Broker.objs):
            return getattr(Broker.objs, key)

    @staticmethod
    def get(orig):
        "return object by origin (repr)"
        return getattr(Broker.objs, orig, None)

    @staticmethod
    def remove(obj):
        "remove object from broker"
        delattr(Broker.objs, rpr(obj))


"interface"


def __dir__():
    return (
        'Broker',
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"status of bots"


from ..broker  import Broker
from ..client  import Client
from ..errors  import Errors


def err(event):
    "show errors."
    nmr = 0
    for bot in Broker.all():
        if 'state' in dir(bot):
            event.reply(str(bot.state))
            nmr += 1
    event.reply(f"status: {nmr} errors: {len(Errors.errors)}")
    for exc in Errors.errors:
        txt = Errors.format(exc)
        for line in txt.split():
            event.reply(line)


"register"


Client.add(err)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105,E1102


"errors"


import io
import traceback


class Errors:

    "Errors"

    errors = []
    filter = []
    output = None
    shown  = []

    @staticmethod
    def add(exc):
        "add an exception"
        excp = exc.with_traceback(exc.__traceback__)
        Errors.errors.append(excp)

    @staticmethod
    def enable(out):
        "enable output"
        Errors.output = out

    @staticmethod
    def format(exc):
        "format an exception"
        res = ""
        stream = io.StringIO(
                             traceback.print_exception(
                                                       type(exc),
                                                       exc,
                                                       exc.__traceback__
                                                      )
                            )
        for line in stream.readlines():
            res += line + "\n"
        return res

    @staticmethod
    def out(exc):
        "check if output function is set."
        if Errors.output is None:
            return
        txt = str(Errors.format(exc))
        Errors.output(txt)

    @staticmethod
    def show():
        "show exceptions"
        for exc in Errors.errors:
            Errors.out(exc)

    @staticmethod
    def skip(txt):
        "check for skipping exceptions"
        for skp in Errors.filter:
            if skp in str(txt):
                return True
        return False


def debug(txt):
    "debug text"
    if Errors.output and not Errors.skip(txt):
        Errors.output(txt)


"interface"


def __dir__():
    return (
        'Errors',
        'debug'
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"fleet"


from ..client  import Client
from ..broker  import Broker
from ..thread  import name


def flt(event):
    "show bots in fleet."
    try:
        event.reply(Broker.all()[int(event.args[0])])
    except (IndexError, ValueError):
        event.reply(",".join([name(x).split(".")[-1] for x in Broker.all()]))


"register"


Client.add(flt)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105,W0614,W0401,W0622


"interface"


from . import broker, client, default, errors, event, handler, object
from . import parser, persist, repeater, thread, timer, utils, workdir


from .broker   import *
from .client   import *
from .default  import *
from .errors   import *
from .event    import *
from .handler  import *
from .object   import *
from .parser   import *
from .persist  import *
from .repeater import *
from .thread   import *
from .timer    import *
from .utils    import *
from .workdir  import *


def __modules__():
    return (
        "broker",
        "client",
        "default",
        "errors",
        "event",
        "handler",
        "interface",
        "object",
        "parser",
        "persist",
        "repeater",
        "thread",
        "timer",
        "utils",
        "workdir"
    )


def __dir__():
    all = []
    all.extend(broker.__dir__())
    all.extend(client.__dir__())
    all.extend(default.__dir__())
    all.extend(errors.__dir__())
    all.extend(event.__dir__())
    all.extend(handler.__dir__())
    all.extend(object.__dir__())
    all.extend(parser.__dir__())
    all.extend(persist.__dir__())
    all.extend(repeater.__dir__())
    all.extend(thread.__dir__())
    all.extend(timer.__dir__())
    all.extend(utils.__dir__())
    all.extend(workdir.__dir__())
    return sorted(all)


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R


"working directory"


import os


from .object import Object, cdir


class Workdir(Object):

    "Workdir"

    workdir = ""

    @staticmethod
    def skel():
        "create directory,"
        cdir(os.path.join(Workdir.workdir, "store", ""))

    @staticmethod
    def store(pth=""):
        "return objects directory."
        return os.path.join(Workdir.workdir, "store", pth)

    @staticmethod
    def strip(pth, nmr=3):
        "reduce to path with directory."
        return os.sep.join(pth.split(os.sep)[-nmr:])

    @staticmethod
    def types():
        "return types stored."
        return os.listdir(Workdir.store())


def __dir__():
    return (
        'Workdir',
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C.R,W0105


"repeater"


from .timer  import Timer
from .thread import launch


class Repeater(Timer):

    "Repeater"

    def run(self):
        launch(self.start)
        super().run()


"interface"


def __dir__():
    return (
        'Repeater',
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"mailbox"


import mailbox
import os
import time


from ..client  import Client
from ..object  import Object, fmt, update
from ..persist import Persist, find, fntime, sync
from ..utils   import laps


MONTH = {
    'Jan': 1,
    'Feb': 2,
    'Mar': 3,
    'Apr': 4,
    'May': 5,
    'Jun': 6,
    'Jul': 7,
    'Aug': 8,
    'Sep': 9,
    'Oct': 10,
    'Nov': 11,
    'Dec': 12
}


class Email(Object):

    "Email"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.text = ""

    def __ok__(self):
        pass

    def __okok__(self):
        pass


def cor(event):
    "search correspondence."
    if not event.args:
        event.reply("cor <email>")
        return
    nrs = -1
    for _fn, email in find("email", {"From": event.args[0]}):
        nrs += 1
        txt = ""
        if len(event.args) > 1:
            txt = ",".join(event.args[1:])
        else:
            txt = "From,Subject"
        form = fmt(email, txt, plain=True)
        lapsed = laps(time.time() - fntime(email.__stp__))
        event.reply(f"{nrs} {form} {lapsed}")


def eml(event):
    "search email from recipient."
    if not event.args:
        event.reply("eml <searchtxtinemail>")
        return
    nrs = -1
    for fnm, obj in find("email"):
        if event.rest in obj.text:
            nrs += 1
            form = fmt(obj, "From,Subject")
            lapsed = laps(time.time() - fntime(fnm))
            event.reply(f"{nrs} {form} {lapsed}")


def mbx(event):
    "scan mailbox."
    if not event.args:
        return
    fnm = os.path.expanduser(event.args[0])
    event.reply(f"reading from {fnm}")
    nrs = 0
    if os.path.isdir(fnm):
        thing = mailbox.Maildir(fnm, create=False)
    elif os.path.isfile(fnm):
        thing = mailbox.mbox(fnm, create=False)
    else:
        return
    try:
        thing.lock()
    except FileNotFoundError:
        pass
    for mail in thing:
        email = Email()
        update(email, mail._headers) # pylint: disable=W0212
        email.text = ""
        for payload in mail.walk():
            if payload.get_content_type() == 'text/plain':
                email.text += payload.get_payload()
        email.text = email.text.replace("\\n", "\n")
        sync(eml)
        nrs += 1
    if nrs:
        event.reply(f"ok {nrs}")


"register"


Client.add(cor)
Client.add(eml)
Client.add(mbx)
Persist.add(Email)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"available modules"


from ..client import Client


from . import __dir__


def mod(event):
    "list modules."
    event.reply(",".join(sorted(__dir__())))


"register"


Client.add(mod)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"utilities"


def laps(seconds, short=True):
    "show elapsed time."
    txt = ""
    nsec = float(seconds)
    if nsec < 1:
        return f"{nsec:.2f}s"
    yea = 365*24*60*60
    week = 7*24*60*60
    nday = 24*60*60
    hour = 60*60
    minute = 60
    yeas = int(nsec/yea)
    nsec -= yeas*yea
    weeks = int(nsec/week)
    nsec -= weeks*week
    nrdays = int(nsec/nday)
    nsec -= nrdays*nday
    hours = int(nsec/hour)
    nsec -= hours*hour
    minutes = int(nsec/minute)
    nsec -= int(minute*minutes)
    sec = int(nsec)
    if yeas:
        txt += f"{yeas}y"
    if weeks:
        nrdays += weeks * 7
    if nrdays:
        txt += f"{nrdays}d"
    if short and txt:
        return txt.strip()
    if hours:
        txt += f"{hours}h"
    if minutes:
        txt += f"{minutes}m"
    if sec:
        txt += f"{sec}s"
    txt = txt.strip()
    return txt


def spl(txt):
    "split comma separated string into a list."
    try:
        res = txt.split(',')
    except (TypeError, ValueError):
        res = txt
    return [x for x in res if x]


"interface"


def __dir__():
    return (
        'laps',
        'spl'
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"persist"


import datetime
import os
import time


from .default import Default
from .object  import Object, fqn, read, search, update, write
from .workdir import Workdir


class Persist(Object):

    "Persist"

    classes = Object()

    @staticmethod
    def add(clz):
        "add class to whitelist."
        name = str(clz).split()[1][1:-2]
        setattr(Persist.classes, name, clz)

    @staticmethod
    def fns(mtc=""):
        "show list of files."
        dname = ''
        pth = Workdir.store(mtc)
        for rootdir, dirs, _files in os.walk(pth, topdown=False):
            if dirs:
                for dname in sorted(dirs):
                    if dname.count('-') == 2:
                        ddd = os.path.join(rootdir, dname)
                        fls = sorted(os.listdir(ddd))
                        for fll in fls:
                            yield Workdir.strip(os.path.join(ddd, fll))

    @staticmethod
    def long(name):
        "match from single name to long name."
        split = name.split(".")[-1].lower()
        res = name
        for named in Persist.classes:
            if split in named.split(".")[-1].lower():
                res = named
                break
        if "." not in res:
            for fnm in Workdir.types():
                claz = fnm.split(".")[-1]
                if fnm == claz.lower():
                    res = fnm
        return res


def fntime(daystr):
    "convert file name to it's saved time."
    daystr = daystr.replace('_', ':')
    datestr = ' '.join(daystr.split(os.sep)[-2:])
    if '.' in datestr:
        datestr, rest = datestr.rsplit('.', 1)
    else:
        rest = ''
    timed = time.mktime(time.strptime(datestr, '%Y-%m-%d %H:%M:%S'))
    if rest:
        timed += float('.' + rest)
    return timed


def find(mtc, selector=None, index=None, deleted=False):
    "find object matching the selector dict."
    clz = Persist.long(mtc)
    nrs = -1
    for fnm in sorted(Persist.fns(clz), key=fntime):
        obj = Default()
        fetch(obj, fnm)
        if not deleted and '__deleted__' in obj:
            continue
        if selector and not search(obj, selector):
            continue
        nrs += 1
        if index is not None and nrs != int(index):
            continue
        yield (fnm, obj)


def fetch(obj, pth):
    "read object from disk."
    pth2 = Workdir.store(pth)
    read(obj, pth2)
    return Workdir.strip(pth)


def ident(obj):
    "return an id for an object."
    return os.path.join(
                        fqn(obj),
                        os.path.join(*str(datetime.datetime.now()).split())
                       )


def last(obj, selector=None):
    "return last object saved."
    if selector is None:
        selector = {}
    result = sorted(
                    find(fqn(obj), selector),
                    key=lambda x: fntime(x[0])
                   )
    res = None
    if result:
        inp = result[-1]
        update(obj, inp[-1])
        res = inp[0]
    return res

def sync(obj, pth=None):
    "sync object to disk."
    if pth is None:
        pth = ident(obj)
    pth2 = Workdir.store(pth)
    write(obj, pth2)
    return pth


"interface"


def __dir__():
    return (
        'Persist',
        'fetch',
        'fntime',
        'find',
        'last',
        'ident',
        'sync',
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"parser"


from .default import Default


def parse_cmd(obj, txt=None):
    "parse a string for a command."
    args = []
    obj.args    = obj.args or []
    obj.cmd     = obj.cmd or ""
    obj.gets    = obj.gets or Default()
    obj.hasmods = obj.hasmod or False
    obj.index   = None
    obj.mod     = obj.mod or ""
    obj.opts    = obj.opts or ""
    obj.result  = obj.reult or []
    obj.sets    = obj.sets or Default()
    obj.txt     = txt or obj.txt or ""
    obj.otxt    = obj.txt
    _nr = -1
    for spli in obj.otxt.split():
        if spli.startswith("-"):
            try:
                obj.index = int(spli[1:])
            except ValueError:
                obj.opts += spli[1:]
            continue
        if "==" in spli:
            key, value = spli.split("==", maxsplit=1)
            if key in obj.gets:
                val = getattr(obj.gets, key)
                value = val + "," + value
            setattr(obj.gets, key, value)
            continue
        if "=" in spli:
            key, value = spli.split("=", maxsplit=1)
            if key == "mod":
                obj.hasmods = True
                if obj.mod:
                    obj.mod += f",{value}"
                else:
                    obj.mod = value
                continue
            setattr(obj.sets, key, value)
            continue
        _nr += 1
        if _nr == 0:
            obj.cmd = spli
            continue
        args.append(spli)
    if args:
        obj.args = args
        obj.txt  = obj.cmd or ""
        obj.rest = " ".join(obj.args)
        obj.txt  = obj.cmd + " " + obj.rest
    else:
        obj.txt = obj.cmd or ""


"interface"


def __dir__():
    return (
        'parse_cmd',
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105,W0212
# ruff: noqa: E402


"main"


import getpass
import os
import pwd
import sys
import termios
import time


from .client  import Client, cmnd
from .default import Default
from .errors  import Errors,debug
from .event   import Event
from .object  import cdir
from .parser  import parse_cmd
from .utils   import spl
from .workdir import Workdir


from . import modules

Cfg          = Default()
Cfg.mod      = "cmd,mod"
Cfg.name     = sys.argv[0].split(os.sep)[-2]
Cfg.dir      = os.path.expanduser(f"~/.{Cfg.name}")
Cfg.pidfile  = os.path.join(Cfg.wd, f"{Cfg.name}.pid")
Workdir.workdir = Cfg.dir


dte = time.ctime(time.time()).replace("  ", " ")
ext = os._exit 


class Console(Client):

    "Console"

    def announce(self, txt):
        "blind announce"

    def callback(self, evt):
        "run and wait for callback to finish."
        Client.callback(self, evt)
        evt.wait(5.0)

    def poll(self):
        "reconstruct event from input."
        evt = Event()
        evt.orig = object.__repr__(self)
        evt.txt = input("> ")
        evt.type = "command"
        return evt

    def say(self, _channel, txt):
        "say text in channel."
        txt = txt.encode('utf-8', 'replace').decode()
        print(txt)


def daemon(pidfile, verbose=False):
    "fork into the background."
    pid = os.fork()
    if pid != 0:
        ext(0)
    os.setsid()
    pid2 = os.fork()
    if pid2 != 0:
        ext(0)
    if not verbose:
        with open('/dev/null', 'r', encoding="utf-8") as sis:
            os.dup2(sis.fileno(), sys.stdin.fileno())
        with open('/dev/null', 'a+', encoding="utf-8") as sos:
            os.dup2(sos.fileno(), sys.stdout.fileno())
        with open('/dev/null', 'a+', encoding="utf-8") as ses:
            os.dup2(ses.fileno(), sys.stderr.fileno())
    os.umask(0)
    os.chdir("/")
    if os.path.exists(pidfile):
        os.unlink(pidfile)
    cdir(os.path.dirname(pidfile))
    with open(pidfile, "w", encoding="utf-8") as fds:
        fds.write(str(os.getpid()))


def init(pkg, modstr, disable=""):
    "start inits in modules."
    mds = []
    for modname in spl(modstr):
        if modname in spl(disable):
            continue
        module = getattr(pkg, modname, None)
        if not module:
            continue
        if "init" in dir(module):
            module.init()
            mds.append(module)
    return mds


def privileges(username):
    "lower privileges."
    pwnam = pwd.getpwnam(username)
    os.setgid(pwnam.pw_gid)
    os.setuid(pwnam.pw_uid)


def wrap(func):
    "restore terminal"
    old2 = None
    try:
        old2 = termios.tcgetattr(sys.stdin.fileno())
    except termios.error:
        pass
    try:
        func()
    except (KeyboardInterrupt, EOFError):
        print("")
    finally:
        if old2:
            termios.tcsetattr(sys.stdin.fileno(), termios.TCSADRAIN, old2)


"runtime"


def main():
    "main code"
    Workdir.skel()
    Errors.enable(print)
    parse_cmd(Cfg, " ".join(sys.argv[1:]))
    result = None
    if 'a' in Cfg.opts:
        Cfg.mod = "," + ",".join(modules.__dir__())
    if "v" in Cfg.opts:
        debug(f"{Cfg.name.upper()} {Cfg.opts.upper()} started {dte}")
    if "h" in Cfg.opts:
        print(__doc__)
        return result
    if "d" in Cfg.opts:
        Cfg.mod = ",".join(modules.__dir__())
        Cfg.user = getpass.getuser()
        daemon(Cfg.pidfile, "v" in Cfg.opts)
        privileges(Cfg.user)
        init(modules, Cfg.mod)
        while 1:
            time.sleep(1.0)
    elif "c" in Cfg.opts:
        init(modules, Cfg.mod)
        csl = Console()
        csl.start()
        while 1:
            time.sleep(1.0)
    elif Cfg.otxt:
        cmnd(Cfg.otxt, print)
    return result


def wrapped():
    "wrapped code"
    wrap(main)
    Errors.show()


if __name__ == "__main__":
    wrapped()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R
# ruff: noqa: F401


"modules"


from . import cmd, dbg, err, flt, fnd, irc, log, mdl, mod, req, rss, tdo, thr, tmr


def __dir__():
    return (
        'cmd',
        'dbg',
        'err',
        'flt',
        'fnd',
        'irc',
        'log',
        'mod',
        'req',
        'rss',
        'tdo',
        'thr',
        'tmr'
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105,W0718


"client"


from .broker  import Broker
from .errors  import Errors
from .event   import Event
from .handler import Handler
from .object  import Object
from .parser  import parse_cmd


class Client(Handler):

    "Client"

    cmds = Object()

    def __init__(self):
        Handler.__init__(self)
        self.register("command", self.command)
        Broker.add(self)

    @staticmethod
    def add(func):
        "add command to client."
        setattr(Client.cmds, func.__name__, func)

    def announce(self, txt):
        "announce text."
        self.raw(txt)

    def command(self, evt):
        "check for and run a command."
        parse_cmd(evt)
        func = getattr(self.cmds, evt.cmd, None)
        if func:
            try:
                func(evt)
                self.show(evt)
            except Exception as ex:
                Errors.add(ex)
        evt.ready()

    def raw(self, txt):
        "raw output."

    def say(self, _channel, txt):
        "say text in a channel."
        self.raw(txt)

    def show(self, evt):
        "show results into a channel."
        for txt in evt.result:
            self.say(evt.channel, txt)


def cmnd(txt, out):
    "do a command using the provided output function."
    clt = Client()
    clt.raw = out
    evn = Event()
    evn.orig = object.__repr__(clt)
    evn.txt = txt
    clt.command(evn)
    evn.wait()
    return evn


"interface"


def __dir__():
    return (
        'Client',
        'cmnd'
    )


__all__ = __dir__()



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105,W0611


"debug"


from ..client import Client


class MyBug(Exception):

    "MyBug"


def dbg(event):
    "raise exception"
    raise MyBug("yo!")


"register"


#Client.add(dbg)



# This file is placed in the Public Domain.
#
# pylint: disable=C,R,W0105


"timer"


import threading
import time


from .thread import launch


class Timer:

    "Timer"

    def __init__(self, sleep, func, *args, thrname=None):
        self.args  = args
        self.func  = func
        self.sleep = sleep
        self.name  = thrname or str(self.func).split()[2]
        self.state = {}
        self.timer = None

    def run(self):
        "run the payload in a thread."
        self.state["latest"] = time.time()
        launch(self.func, *self.args)

    def start(self):
        "start timer."
        timer = threading.Timer(self.sleep, self.run)
        timer.name   = self.name
        timer.daemon = True
        timer.sleep  = self.sleep
        timer.state  = self.state
        timer.func   = self.func
        timer.state["starttime"] = time.time()
        timer.state["latest"]    = time.time()
        timer.start()
        self.timer   = timer

    def stop(self):
        "stop timer."
        if self.timer:
            self.timer.cancel()


"interface"


def __dir__():
    return (
        'Timer',
    )


__all__ = __dir__()
